classdef asymmetricRolling
    
    properties (Constant)
        ht = @(x, rt, delta) (1-rt) + (1+delta^2*rt^2)/(2*rt*delta^2) - sqrt((1+delta^2*rt^2)^2/(4*rt^2*delta^2) - (x-1).^2)/delta;
        hb = @(x, rb, delta) -asymmetricRolling.ht(x,rb,delta);
        htprime = @(x,rt,delta) (x-1)./(sqrt((1+delta^2*rt^2)^2/(4*rt^2*delta^2) - (x-1).^2)*delta);
        hbprime = @(x,rb,delta) -asymmetricRolling.htprime(x,rb,delta);
        htdprime = @(x,rt,delta) ((1+delta^2*rt^2)^2/(4*rt^2*delta^2) - (x-1).^2).^(-1/2)/delta + (x-1).^2.*((1+delta^2*rt^2)^2/(4*rt^2*delta^2) - (x-1).^2).^(-3/2)/delta;
        hbdprime = @(x,rb,delta) -asymmetricRolling.htdprime(x,rb,delta);
        httprime = @(x,rt,delta) 6*(x-1).*((1+delta^2*rt^2)^2/(4*rt^2*delta^2) - (x-1).^2).^(-3/2)/delta + 6*(x-1).^3.*((1+delta^2*rt^2)^2/(4*rt^2*delta^2) - (x-1).^2).^(-5/2)/delta;
        hbtprime = @(x,rb,delta) -asymmetricRolling.httprime(x,rb,delta);

        dh = @(x,rt,rb, delta) asymmetricRolling.ht(x, rt, delta) - asymmetricRolling.hb(x, rb, delta);
        dhprime = @(x,rt,rb, delta) asymmetricRolling.htprime(x, rt, delta) - asymmetricRolling.hbprime(x,rb, delta);
        dhdprime = @(x,rt,rb, delta) asymmetricRolling.htdprime(x, rt, delta) - asymmetricRolling.hbdprime(x,rb, delta);
        dhtprime = @(x,rt,rb, delta) asymmetricRolling.httprime(x, rt, delta) - asymmetricRolling.hbtprime(x,rb, delta);
    end

    methods(Static)
        function [res, grid, param, f, resD0, resD2] = runAsymmetricRolling(paramIn)
            param = asymmetricRolling.cleanParam(paramIn);
            %Asymptotic calculations
            [res,res0,res1,res2,x,y]  = asymmetricRolling.calculateScaledSolution(param); 
            %Redimensionalise
            [res, resD0, resD1, resD2, grid] = asymmetricRolling.dimensionalise(res0, res1, res2, x,y,res.xn,param);
            %Post processing roll forces
            [res.rollPressure, res.rollShear, res.rollForce, res.rollTorque, res.rollForceHorizontal] = asymmetricRolling.rollPressure(res, grid, param);
            [resD0.rollPressure, resD0.rollShear, resD0.rollForce, resD0.rollTorque, resD0.rollForceHorizontal] = asymmetricRolling.rollPressure(resD0, grid, param);
            [resD2.rollPressure, resD2.rollShear, resD2.rollForce, resD2.rollTorque, resD2.rollForceHorizontal] = asymmetricRolling.rollPressure(resD2, grid, param);
            f = asymmetricRolling.horizontalForce(res.sxx-res.p, grid, param, 1:param.nx); %Horizontal force on workpiece through roll gap
            %Check solution validity
            if(any(res.rollPressure < -0.0))
                warning('High end tensions has resulted in a negative roll pressure');
            end
        end
        function param = cleanParam(paramIn)
                param.nx = paramIn.horizontalElements_+1; %Number of nodes in the horizontal direction
                param.ny = 2*paramIn.verticalElements_+1; %Number of nodes in the vertical direction
                param.odeAbsTol = 1e-14;

                %Dimensional geometric quantities.
                param.Tin = paramIn.tensionIn_N; %Tension at the roll gap entrance in N
                param.Tout = paramIn.tensionOut_N; %Tension at the roll gap exit in N        
                param.wT = paramIn.workpieceThickness_m/2; %Workpiece half-thickness in m
                param.wr = param.wT*paramIn.workpieceReduction_fraction; %Workpiece half-reduction per pass as a fraction.
                param.Rt = paramIn.rollerRadius_m_top;
                param.Rb = paramIn.rollerRadius_m_bottom;
                param.rt = 2*param.wr * (param.Rb - param.wr)/(param.Rb - param.wr + param.Rt - param.wr);
                param.rb = 2*param.wr * (param.Rt - param.wr)/(param.Rb - param.wr + param.Rt - param.wr);
                param.wl = sqrt(2*param.Rb*param.rb - param.rb^2); % in mm %sqrt(param.rR^2 - (param.rR-param.wr)^2)
                %Non-dimensional geometric quantities
                param.rt = param.rt/param.wT;
                param.rb = param.rb/param.wT;        
                param.r = param.wr/param.wT;
                param.delta = param.wT/param.wl;

                %Roll Speeds
                if isfield(paramIn, 'rotationalSpeed_rps_top') && isfield(paramIn, 'rotationalSpeed_rps_bottom')
                    param.omegat = paramIn.rotationalSpeed_rps_top;
                    param.omegab = paramIn.rotationalSpeed_rps_bottom;
                    param.Ut = 2*pi*param.Rt * param.omegat;
                    param.Ub = 2*pi*param.Rb * param.omegab;
                elseif isfield(paramIn, 'rollerSpeed_mps_top') && isfield(paramIn, 'rollerSpeed_mps_bottom')
                    param.Ut = paramIn.rollerSpeed_mps_top;
                    param.Ub = paramIn.rollerSpeed_mps_bottom;
                    param.omegat = param.Ut/(2*pi*param.Rt);
                    param.omegab = param.Ub/(2*pi*param.Rb);
                else
                    error('incorrect input parameters in paramIn');
                end
                
                %Yield and friciotns
                param.k = paramIn.shearYieldStress_Pa;
                if isfield(paramIn, 'beta_top') && isfield(paramIn, 'beta_bottom')
                    param.mut = paramIn.beta_top;
                    param.mub = paramIn.beta_bottom;
                elseif isfield(paramIn, 'frictionCoefficient_top') && isfield(paramIn, 'frictionCoefficient_bottom')
                    param.mut = paramIn.frictionCoefficient_top;
                    param.mub = paramIn.frictionCoefficient_bottom;
                else
                    error('incorrect input parameters in paramIn');
                end
                param.beta = param.mub; %not scaled to one
                param.fric = @(x,xnTop,xnBot)(param.mut*((x<=xnTop)-(x>xnTop)) - param.mub*((x>=xnBot)-(x<xnBot)))/param.beta;
                param.fricTop = @(x,xnTop)param.mut*((x<=xnTop)-(x>xnTop))/param.beta;
                param.fricBot = @(x,xnBot)param.mub*((x>=xnBot)-(x<xnBot))/param.beta;

                % Nondimensionalisations
                param.S0 = param.k;
                param.tau0 = param.beta*param.k;
                param.p0 = param.beta/param.delta*param.k;
                param.u0 = param.Ut; %Redifined later once velocity distribution is known.

                %End conditions 
                param.Tin0 = param.Tin/(param.S0*2*param.wT);%Assume leading order tension and 'non-dimensionalise'
                param.Tin1 = 0;
                param.Tout0 = param.Tout/(param.S0*2*param.wT);
                param.Tout1 = 0;
                param.deltaT = (param.Tin0+param.delta*param.Tin1) - (param.Tout0+param.delta*param.Tout1);
        end
        function [res, res0, res1, res2, x, y] = calculateScaledSolution(param)
            %O(delta) are all zero
            [x,y] = meshgrid(linspace(0,1,param.nx), linspace(-1,1,param.ny));
            for i=1:param.nx
                y(:,i) = linspace(asymmetricRolling.hb(x(1,i),param.rb,param.delta), asymmetricRolling.ht(x(1,i),param.rt,param.delta), param.ny);
            end; clear('xi')
            t = cputime;
            [res0, res.xn, c2] = asymmetricRolling.calculateO0(x,y,param);
            res0.cpuTime = t-cputime;
            t = cputime;
            res1 = asymmetricRolling.calculateO1(x);
            res1.cpuTime = t-cputime;
            t = cputime;
            res2 = asymmetricRolling.calculateO2(x,y,param, res.xn, res0, c2);
            res2.cpuTime = t-cputime;
            
            res.p = res0.p + param.delta*res1.p + param.delta^2*res2.p;
            res.sxx = res0.sxx + param.delta*res1.sxx + param.delta^2*res2.sxx;
            res.syy = -res.sxx;
            res.sxy = res0.sxy + param.delta*res1.sxy + param.delta^2*res2.sxy;
            res.vx = res0.vx + param.delta*res1.vx + param.delta^2*res2.vx;
            res.vy = res0.vy + param.delta*res1.vy + param.delta^2*res2.vy;
            res.lambda = res0.lambda + param.delta*res1.lambda + param.delta^2*res2.lambda;
            res.cpuTime = res0.cpuTime+res1.cpuTime+res2.cpuTime;
        end
        function [resD, resD0, resD1, resD2, grid] = dimensionalise(res0, res1, res2, x,y,xn,param)
            %scale velocity by entrance velocity
            ht = asymmetricRolling.ht(xn(1), param.rt,param.delta);
        	vx = asymmetricRolling.vx0(xn(1), param);
        	vy = asymmetricRolling.vy0(xn(1), ht, param);
            param.u0 = param.Ut/sqrt(vx^2 + vy^2);
            
            resD0.vx = param.u0 * res0.vx;
            resD1.vx = param.u0 * param.delta*res1.vx;
            resD2.vx = param.u0 * param.delta^2*res2.vx;
            resD.vx = resD0.vx + resD1.vx + resD2.vx;
            resD0.vy = param.u0 * res0.vy;
            resD1.vy = param.u0 * param.delta*res1.vy;
            resD2.vy = param.u0 * param.delta^2*res2.vy;
            resD.vy = resD0.vy + resD1.vy + resD2.vy;
            resD0.p = param.p0 * res0.p;
            resD1.p = param.p0 * param.delta*res1.p;
            resD2.p = param.p0 * param.delta^2*res2.p;
            resD.p = resD0.p + resD1.p + resD2.p;
            resD0.syy = param.S0 * res0.syy;
            resD1.syy = param.S0 * param.delta*res1.syy;
            resD2.syy = param.S0 * param.delta^2*res2.syy;
            resD.syy = resD0.syy + resD1.syy + resD2.syy;
            resD0.sxx = param.S0 * res0.sxx;
            resD1.sxx = param.S0 * param.delta*res1.sxx;
            resD2.sxx = param.S0 * param.delta^2*res2.sxx;
            resD.sxx = resD0.sxx + resD1.sxx + resD2.sxx;
            resD0.sxy = param.tau0 * res0.sxy;
            resD1.sxy = param.tau0 * param.delta*res1.sxy;
            resD2.sxy = param.tau0 * param.delta^2*res2.sxy;
            resD.sxy = resD0.sxy + resD1.sxy + resD2.sxy;
            resD0.lambda = param.u0/(param.S0*param.wl) * res0.lambda;
            resD1.lambda = param.u0/(param.S0*param.wl) * param.delta * res0.lambda;
            resD2.lambda = param.u0/(param.S0*param.wl) * param.delta^2 * res0.lambda;
            resD.lambda = resD0.lambda + resD1.lambda + resD2.lambda;
            
            resD0.cpuTime = res0.cpuTime;
            resD2.cpuTime = res2.cpuTime;
            resD.cpuTime0 = res0.cpuTime;
            resD.cpuTime2 = res2.cpuTime;
            resD.cpuTime = res0.cpuTime+res2.cpuTime;

            resD.xn = xn .* param.wl;
            resD0.xn = resD.xn;
            resD1.xn = resD.xn;
            resD2.xn = resD.xn;

            grid.X = x .* param.wl;
            grid.Y = y .* param.wT;
            grid.ht = asymmetricRolling.ht(x,param.rt,param.delta).*param.wT;
            grid.hb = asymmetricRolling.hb(x,param.rb,param.delta).*param.wT;
            grid.htprime = asymmetricRolling.htprime(x,param.rt,param.delta).*param.wT; 
            grid.hbprime = asymmetricRolling.hbprime(x,param.rb,param.delta).*param.wT;
        end

%% O(1) functions
        function [res, xn, c2] = calculateO0(x,y,param)
            res.sxx = asymmetricRolling.sxx0(x);
            res.syy = -res.sxx;
            res.vx = asymmetricRolling.vx0(x,param);
            res.vy = asymmetricRolling.vy0(x,y,param);
            res.lambda = asymmetricRolling.lambda0(x,res.sxx,param);
            [p0sym, c2sym, xnsym] = asymmetricRolling.p0c2Sym(param);
            xn = asymmetricRolling.xn(p0sym, c2sym, xnsym, param);
            [res.p, c2] = asymmetricRolling.p0c2(x,p0sym,c2sym,param,xn);
            res.sxy = asymmetricRolling.sxy0(x,y,res.p,res.sxx,param,xn);
        end
        function sxx = sxx0(x)
            sxx = ones(size(x));
        end
        function vx = vx0(x, param)
            vx = 2.0./asymmetricRolling.dh(x, param.rt, param.rb, param.delta);
        end
        function vy = vy0(x, y, param)
            dh = asymmetricRolling.dh(x, param.rt, param.rb, param.delta); 
            dhprime = asymmetricRolling.dhprime(x, param.rt, param.rb, param.delta);
            ht = asymmetricRolling.ht(x, param.rt,param.delta);
            hb = asymmetricRolling.hb(x, param.rb,param.delta);
            htprime = asymmetricRolling.htprime(x, param.rt, param.delta);
            hbprime = asymmetricRolling.hbprime(x, param.rb, param.delta);
            vy = (2.0./dh.^2 .* (ht.*hbprime - hb .* htprime + y.*dhprime));
        end
        function lambda = lambda0(x, sxx0, param)
            dhprime = asymmetricRolling.dhprime(x, param.rt, param.rb, param.delta);
            dh = asymmetricRolling.dh(x, param.rt, param.rb, param.delta);
            lambda = -2*(dhprime)./(sxx0.*dh.^2);
        end   
        function [K, dK, ddK] = K(x,p0,sxx0,param,xn)
            beta = param.beta/param.delta;
            ht = asymmetricRolling.ht(x,param.rt,param.delta);
            hb = asymmetricRolling.hb(x,param.rb,param.delta);
            deltaH = ht - hb;
            dht = asymmetricRolling.htprime(x,param.rt,param.delta);
            dhb = asymmetricRolling.hbprime(x,param.rb,param.delta);
            ddeltaH = dht - dhb;
            ddht = asymmetricRolling.htdprime(x,param.rt,param.delta);
            ddhb = asymmetricRolling.hbdprime(x,param.rb,param.delta);
            dddeltaH = ddht - ddhb;
            dddht = asymmetricRolling.httprime(x,param.rt,param.delta);
            dddhb = asymmetricRolling.hbtprime(x,param.rb,param.delta);
            gammat = param.fricTop(x,xn(1));
            gammab = param.fricBot(x,xn(2));
            [dp0, ddp0] = asymmetricRolling.dp0(x, p0, param, xn);
            K = 1./deltaH.*((gammab.*ht-gammat.*hb).*(beta*p0+sxx0) + 2/beta*sxx0.*(ht.*dhb - hb.*dht));
            dK = 1./deltaH.*((gammab.*dht-gammat.*dhb).*(beta.*p0+sxx0) + (gammab.*ht-gammat.*hb).*beta.*dp0 + 2/beta*sxx0.*(ht.*ddhb-hb.*ddht))-ddeltaH./deltaH.*K;
            ddK = 1./deltaH.*(-2*ddeltaH.*dK - dddeltaH.*K + (gammab.*ddht-gammat.*ddhb).*(beta*p0+sxx0) + 2*beta*(gammab.*dht - gammat.*dhb).*dp0 + (gammab.*ht - gammat.*hb).*ddp0 + 2/beta*sxx0.*(ht.*dddhb+dht.*ddhb - ddht.*dhb-dddht.*hb));
        end
        function [p0sym,c2sym,xnsym] = p0c2Sym(param)
            %y=0;%neutral point found for continutiy along the centreline
            beta = param.beta/param.delta;
            xini =[0.0,1.0];
            xnini = [0.5,0.5];
            ht = asymmetricRolling.ht(xini,param.rt,param.delta);
            hb = asymmetricRolling.hb(xini,param.rb,param.delta);
            deltaH = ht - hb;
            %param.Tin0 = param.Tin/(param.S0*2*param.wT); %This is the nondimensionalisation to use
            sxx0ini = asymmetricRolling.sxx0(xini);
            p0ini = [(1 - param.Tin0/deltaH(1) )*1/beta,...
                     (1 - param.Tout0/deltaH(2))*1/beta];
            [dp0ini, ddp0ini] = asymmetricRolling.dp0(xini,p0ini,param,[0.5, 0.5]);            
            [Kini, dKini,~] = asymmetricRolling.K(xini, p0ini,sxx0ini,param,xnini);
            c2ini = -1./deltaH.*(ddp0ini.*(ht.^3-hb.^3)/6 + dKini.*(ht.^2-hb.^2)/2)...
                        - beta./(deltaH.*sxx0ini).*(dp0ini.^2.*(ht.^3-hb.^3)/3 + dp0ini.*Kini.*(ht.^2-hb.^2) + Kini.^2.*deltaH);
            xpoints = linspace(0,1,param.nx*3);
            [xode, pode] = ode45(@(x,p)asymmetricRolling.dp0c2(x, p, param, [inf, inf]), xpoints, [p0ini(1), c2ini(1)]);
            p0in = @(x)interp1(xode, pode(:,1), x);
            c2in = @(x)interp1(xode, pode(:,2), x);
            [xode, pode] = ode45(@(x,p)asymmetricRolling.dp0c2(x, p, param, [-inf, -inf]), fliplr(xpoints), [p0ini(2), c2ini(2)]);
            p0out = @(x)interp1(xode, pode(:,1), x);
            c2out = @(x)interp1(xode, pode(:,2), x);
            xnsym = fminbnd(@(x)abs((p0in(x)+c2in(x)*param.delta^2)-(p0out(x)+c2out(x)*param.delta^2)), 0.0, 1.0);
            function res = minIntp(x,fin,fout,xnsym)
                res=[];
                for i=1:size(x,2)
                    if x(:,i) < xnsym
                        res(:,i) = fin(x(:,i));
                    else
                        res(:,i) = fout(x(:,i));
                    end
                end
            end
            p0sym = @(x)minIntp(x,p0in,p0out,xnsym);
            c2sym = @(x)minIntp(x,c2in,c2out,xnsym);
        end
        function xn = xn(p0sym, c2sym, xnsym, param)
            %This function determines the top and bottom neutral points using numerical searching.
            function [xn, warn] = findSecondNeutralPoint(xn1, p0sym, c2sym, xnsym, param)
                %Locates xn2 to ensure continuity of beta*(p0 + delta^2 * p2) + (sxx0 + delta^2 sxx2) along y=0.
                dpCross = @(x,p,topLessBot)asymmetricRolling.dp0c2Cross(x, p, param, topLessBot);
                pini = [p0sym(xn1), c2sym(xn1)];
                opts=odeset('Events',@(x,p0c2)asymmetricRolling.odeSolverTermination(x,p0c2,@(x)[p0sym(x),c2sym(x)],param),'AbsTol', param.odeAbsTol);
                warn = '';
                if xn1<xnsym
                    [xode,odeResult,xn2,~,~] = ode45(@(x,p)dpCross(x,p,true), [xn1, 1.0], pini, opts);
                    if isempty(xn2) || abs(xn1-xn2(end))<1e-14
                        xn2=1.0;
                        warn = 'over';
                    end
                elseif xn1>xnsym
                    [~,odeResult,xn2,~,~] = ode45(@(x,p)dpCross(x,p,false), [xn1, 0.0], pini, opts);
                    if isempty(xn2) || abs(xn1-xn2(end))<1e-14
                        xn2=0.0;
                        warn = 'under';
                    end
                else
                    xn2 = xn1;
                end
                xn = [xn1, xn2(end)];
            end
            function v1v2 = v1v2(xn1,p0sym,c2sym,xnsym,param)
                %Determines the velocity ratios (plus penalty) given the first neutral point.
                [xnInner, warn] = findSecondNeutralPoint(xn1,p0sym,c2sym,xnsym,param);
                penalty = 0;%penalty to the objective function to force the search within the roll gap.
                if strcmp(warn, 'over')
                    penalty = -1e12;
                elseif strcmp(warn, 'under')
                    penalty = 1e12;
                end
                velocityMagnitude = @(x,y,param) sqrt((asymmetricRolling.vx0(x,param)).^2 + (param.delta*asymmetricRolling.vy0(x,y,param)).^2);
                v1v2 = velocityMagnitude(xnInner(1), asymmetricRolling.ht(xnInner(1),param.rt,param.delta),param)/velocityMagnitude(xnInner(2), asymmetricRolling.hb(xnInner(2),param.rb,param.delta),param) + penalty;
            end
            xn1 = fminbnd(@(xn1)abs(v1v2(xn1,p0sym, c2sym, xnsym, param) - param.Ut/param.Ub),0.0,1.0);              
            xn = findSecondNeutralPoint(xn1,p0sym, c2sym, xnsym, param);
        end
        function [p0, c2] = p0c2(x,p0sym,c2sym,param,xn)
            %c2 is also found here as the same numerical integrations are required.
            xrow = x(1,:); %Assume grid is regular in x.
            fsym = @(x)[p0sym(x);c2sym(x)];
            vert = size(x,1);
            opts = odeset('AbsTol', param.odeAbsTol);            
            p0c2 = [fsym(xrow(xrow<=min(xn))), asymmetricRolling.ode45Wrapper(@(x,p)asymmetricRolling.dp0c2(x,p,param,xn), xrow(and(xrow>min(xn), xrow<max(xn))), min(xn), fsym(min(xn)), opts), fsym(xrow((xrow>=max(xn)) + (xrow>min(xn)) == 2))];
            p0 = repmat(p0c2(1,:), vert, 1);
            c2 = repmat(p0c2(2,:), vert, 1);
        end
        function sxy = sxy0(x, y, p0, sxx0, param, xn)
            dp0 = asymmetricRolling.dp0(x,p0,param,xn);
            K = asymmetricRolling.K(x,p0,sxx0,param,xn);
            sxy = dp0.*y + K;
        end
%% O(1) derivative functions (used for ODE solving)
        function dp0c2 = dp0c2(x, p, param, xn)
            %p = [p0, c2];
            ht = asymmetricRolling.ht(x,param.rt,param.delta);
            hb = asymmetricRolling.hb(x,param.rb,param.delta);
            deltaH = ht - hb;
            p0 = p(1,:);
            c2 = p(2,:);
            sxx0 = asymmetricRolling.sxx0(x);
            sxy0(1,:) = asymmetricRolling.sxy0(x,ht,p0,sxx0,param,xn);
            sxy0(2,:) = asymmetricRolling.sxy0(x,hb,p0,sxx0,param,xn);
            sxx2(1,:) = asymmetricRolling.sxx2(sxx0, sxy0(1,:), param);
            sxx2(2,:) = asymmetricRolling.sxx2(sxx0, sxy0(2,:), param);
            p2(1,:) = asymmetricRolling.p2(x,ht,p0,sxx0,sxx2(1,:),c2,param,xn);
            p2(2,:) = asymmetricRolling.p2(x,hb,p0,sxx0,sxx2(2,:),c2,param,xn);
            [sxybctop, sxybcbot] = asymmetricRolling.sxy2BC(x,[sxx0;sxx0],[p0;p0],sxy0,sxx2,p2,param,xn);
            
            dp0 = asymmetricRolling.dp0(x,p0,param,xn);
            dc2 = 1./deltaH .* (sxybctop - sxybcbot - (asymmetricRolling.sxy2minus(x,ht,sxx0,p0,param,xn)-asymmetricRolling.sxy2minus(x,hb,sxx0,p0,param,xn)));
            dp0c2 = [dp0; dc2];
        end
        function dp0c2 = dp0c2Cross(x, p, param, top)
            %top is true if the top(first) neutral point is less than the bottom (second).
            if top
                xn = [-inf, inf];
            else
                xn = [inf, -inf];
            end 
            dp0c2 = asymmetricRolling.dp0c2(x,p,param,xn);
        end
        function [dp0, ddp0, dddp0] = dp0(x,p0,param,xn)
            beta = param.beta/param.delta;
            deltaGamma = param.fric(x, xn(1), xn(2));
            deltaH = asymmetricRolling.dh(x, param.rt, param.rb, param.delta);
            ddeltaH = asymmetricRolling.dhprime(x, param.rt, param.rb, param.delta);
            dddeltaH = asymmetricRolling.dhdprime(x, param.rt, param.rb, param.delta);
            ddddeltaH = asymmetricRolling.dhtprime(x,param.rt,param.rb, param.delta);
            sxx0 = asymmetricRolling.sxx0(x);
            dp0 = 1./deltaH .*(deltaGamma.*(beta*p0 + sxx0) + 2/beta*ddeltaH.*sxx0);
            ddp0 = dp0.*(beta*deltaGamma./deltaH - ddeltaH./deltaH) + 2/beta*sxx0.*dddeltaH./deltaH;
            dddp0 = (beta*deltaGamma./deltaH - ddeltaH./deltaH).*(ddp0 - dp0.*ddeltaH./deltaH) - dp0.*dddeltaH./deltaH + 2/beta.*sxx0./deltaH.*(ddddeltaH - ddeltaH.*dddeltaH./deltaH);
        end

%% O(delta) functions
        function [res] = calculateO1(x)
            %O(delta) are all zero
            res.sxx = 0.0*x;
            res.syy = 0.0*x;
            res.vx = 0.0*x;
            res.vy = 0.0*x;
            res.lambda = 0.0*x;
            res.p = 0.0*x;
            res.sxy = 0.0*x;
        end

%% O(delta^2) functions
        function res = calculateO2(x,y,param,xn,res0, c2)
            res.sxx = asymmetricRolling.sxx2(res0.sxx, res0.sxy, param);
            res.syy = -res.sxx;
            res.vx = asymmetricRolling.vx2(x, y, res0.p, res0.sxx, param, xn);
            res.vy = asymmetricRolling.vy2(x, y, res0.p, res0.sxx, param, xn);
            res.lambda = asymmetricRolling.lambda2(x, y, res0.p, res0.sxx, res0.lambda, res.sxx, res.vx, param, xn);
            res.p = asymmetricRolling.p2(x, y, res0.p, res0.sxx, res.sxx, c2, param, xn);
            res.sxy = asymmetricRolling.sxy2(x, y, res0.sxx, res0.p, res0.sxy, res.sxx, c2, res.p, param, xn);
        end
        function sxx = sxx2(sxx0, sxy0, param)
            beta = param.beta/param.delta;
            sxx = -beta^2 * sxy0.^2 ./ (2*sxx0);
        end
        function vx = vx2(x, y, p0, sxx0, param, xn)
            beta = param.beta/param.delta;
            hb = asymmetricRolling.hb(x,param.rb,param.delta);
            ht = asymmetricRolling.ht(x,param.rt,param.delta);
            deltaH = ht-hb;
            dhb = asymmetricRolling.hbprime(x,param.rb,param.delta);
            dht = asymmetricRolling.htprime(x,param.rt,param.delta);
            ddeltaH = dht - dhb;
            ddhb = asymmetricRolling.hbdprime(x,param.rb,param.delta);
            ddht = asymmetricRolling.htdprime(x,param.rt,param.delta);
            dddeltaH = ddht - ddhb;
            dp0 = asymmetricRolling.dp0(x,p0,param,xn);
            K = asymmetricRolling.K(x,p0,sxx0,param,xn);
            vx = 2./deltaH.^2.*((y-(ht.^2-hb.^2)./(2.*deltaH)).*(2*beta*ddeltaH.*K - (ht.*ddhb-hb.*ddht) + 2.*ddeltaH./deltaH.*(ht.*dhb-hb.*dht))...
                                    + (y.^2/2 - (ht.^3-hb.^3)./(6*deltaH)).*(2*beta*ddeltaH.*dp0 - dddeltaH + 2.*ddeltaH.^2./deltaH));
        end
        function vyminus = vy2minus(x, y, p0, sxx0, param, xn)
            beta = param.beta/param.delta;
            ht = asymmetricRolling.ht(x,param.rt,param.delta);
            hb = asymmetricRolling.hb(x,param.rb,param.delta);
            deltaH = ht - hb;
            dht = asymmetricRolling.htprime(x,param.rt,param.delta);
            dhb = asymmetricRolling.hbprime(x,param.rb,param.delta);
            ddeltaH = dht - dhb;
            ddht = asymmetricRolling.htdprime(x,param.rt,param.delta);
            ddhb = asymmetricRolling.hbdprime(x,param.rb,param.delta);
            dddeltaH = ddht - ddhb;
            dddht = asymmetricRolling.httprime(x,param.rt,param.delta);
            dddhb = asymmetricRolling.hbtprime(x,param.rb,param.delta);
            ddddeltaH = dddht-dddhb;
            [dp0, ddp0] = asymmetricRolling.dp0(x, p0, param, xn);
            [K,dK] = asymmetricRolling.K(x,p0,sxx0,param,xn);
            integralu2 = 2./deltaH.^2.*((2*beta*ddeltaH.*K - (ht.*ddhb-hb.*ddht) + 2.*ddeltaH./deltaH.*(ht.*dhb-hb.*dht)).*(y.^2/2-(ht.^2-hb.^2)./(2.*deltaH).*y)...
                                    + (2*beta*ddeltaH.*dp0 - dddeltaH + 2.*ddeltaH.^2./deltaH).*(y.^3/6 - (ht.^3-hb.^3)./(6*deltaH).*y));
            vyminus = 2*ddeltaH./deltaH.*integralu2 - 2./deltaH.^2.*((2*beta*dddeltaH.*K + 2*beta.*ddeltaH.*dK - (ht.*dddhb+dht.*ddhb-dht.*dhb-dddht.*hb) + (2.*dddeltaH./deltaH-2*ddeltaH.^2./deltaH.^2).*(ht.*dhb-hb.*dht) + 2*ddeltaH./deltaH.*(ht.*ddhb-hb.*ddht)).*(y.^2/2-(ht.^2-hb.^2)./(2.*deltaH).*y)...
                                        +(2*beta*ddeltaH.*K - (ht.*ddhb-hb.*ddht) + 2.*ddeltaH./deltaH.*(ht.*dhb-hb.*dht)).*((ht.^2-hb.^2)./(2.*deltaH.^2).*ddeltaH - (ht.*dht-hb.*dhb)./deltaH).*y...
                                    +(2*beta*dddeltaH.*dp0+2*beta*ddeltaH.*ddp0-ddddeltaH + 4*ddeltaH.*dddeltaH./deltaH - 2*ddeltaH.^3./deltaH.^2).*(y.^3/6-(ht.^3-hb.^3)./(6.*deltaH).*y)...
                                        +(2*beta*ddeltaH.*dp0 - dddeltaH + 2.*ddeltaH.^2./deltaH).*((ht.^3-hb.^3)./(6.*deltaH.^2).*ddeltaH - (ht.^2.*dht-hb.^2.*dhb)./(2.*deltaH)).*y);
        end
        function vy = vy2(x,y,p0,sxx0,param,xn)
            ht = asymmetricRolling.ht(x,param.rt,param.delta);
            c1 = asymmetricRolling.htprime(x,param.rt,param.delta).*asymmetricRolling.vx2(x,ht,p0,sxx0,param,xn)-asymmetricRolling.vy2minus(x,ht,p0,sxx0,param,xn);
            vy = asymmetricRolling.vy2minus(x,y,p0,sxx0,param,xn) + c1;
        end
        function lambda = lambda2(x, y, p0, sxx0, lambda0, sxx2, vx2, param, xn)
            beta = param.beta/param.delta;
            ht = asymmetricRolling.ht(x,param.rt,param.delta);
            hb = asymmetricRolling.hb(x,param.rb,param.delta);
            deltaH = ht - hb;
            dht = asymmetricRolling.htprime(x,param.rt,param.delta);
            dhb = asymmetricRolling.hbprime(x,param.rb,param.delta);
            ddeltaH = dht - dhb;
            ddht = asymmetricRolling.htdprime(x,param.rt,param.delta);
            ddhb = asymmetricRolling.hbdprime(x,param.rb,param.delta);
            dddeltaH = ddht - ddhb;
            dddht = asymmetricRolling.httprime(x,param.rt,param.delta);
            dddhb = asymmetricRolling.hbtprime(x,param.rb,param.delta);
            ddddeltaH = dddht-dddhb;          
            [dp0, ddp0] = asymmetricRolling.dp0(x, p0, param, xn);
            [K, dK] = asymmetricRolling.K(x, p0,sxx0,param,xn);
            dvx2dx = - 2*ddeltaH./deltaH.*vx2 + 2./deltaH.^2.*((beta*dddeltaH.*K + beta.*ddeltaH.*dK - (ht.*dddhb+dht.*ddhb-dht.*dhb-dddht.*hb) + (2.*dddeltaH./deltaH-2*ddeltaH.^2./deltaH.^2).*(ht.*dhb-hb.*dht) + 2*ddeltaH./deltaH.*(ht.*ddhb-hb.*ddht)).*(y-(ht.^2-hb.^2)./(2.*deltaH))...
                                            +(beta*ddeltaH.*K - (ht.*ddhb-hb.*ddht) + 2.*ddeltaH./deltaH.*(ht.*dhb-hb.*dht)).*((ht.^2-hb.^2)./(2.*deltaH.^2).*ddeltaH - (ht.*dht-hb.*dhb)./deltaH)...
                                        +(beta*dddeltaH.*dp0+beta*ddeltaH.*ddp0-ddddeltaH + 4*ddeltaH.*dddeltaH./deltaH - 2*ddeltaH.^3./deltaH.^2).*(y.^2/2-(ht.^3-hb.^3)./(6.*deltaH))...
                                            +(beta*ddeltaH.*dp0 - dddeltaH + 2.*ddeltaH.^2./deltaH).*((ht.^3-hb.^3)./(6.*deltaH.^2).*ddeltaH - (ht.^2.*dht-hb.^2.*dhb)./(2.*deltaH)));
            lambda = (dvx2dx - lambda0 .* sxx2)./sxx0;
        end
        function [p] = p2(x, y, p0, sxx0, sxx2, c2, param, xn)
            beta = param.beta/param.delta;
            [~, ddp0] = asymmetricRolling.dp0(x, p0, param, xn);
            [~, dK] = asymmetricRolling.K(x, p0,sxx0,param,xn);
            p = ddp0.*y.^2/2 + dK.*y - sxx2/beta + c2;
        end
        function sxy = sxy2minus(x, y, sxx0, p0, param, xn)
            beta = param.beta/param.delta;
            [dp0, ddp0, dddp0] = asymmetricRolling.dp0(x, p0, param, xn);
            [K, dK, ddK] = asymmetricRolling.K(x,p0,sxx0,param,xn);
            sxy = y.^3/3.*(dddp0/2 + 2*beta./sxx0.*dp0.*ddp0) + y.^2/2.*(ddK + 2*beta./sxx0.*(dp0.*dK + ddp0.*K)) + y.*(2*beta./sxx0.*dK.*K);
        end
        function sxy = sxy2(x, y, sxx0, p0, sxy0, sxx2, c2, p2, param, xn)
            xrow = x(1,:); %Assume grid in x is invariant in y
            vert = size(x,1);
            dp0c2 = asymmetricRolling.dp0c2(xrow,[p0(1,:); c2],param,xn);
            [sxybc, ~] = asymmetricRolling.sxy2BC(xrow,[sxx0(end,:);sxx0(1,:)], [p0(end,:);p0(1,:)], [sxy0(end,:);sxy0(1,:)], [sxx2(end,:);sxx2(1,:)], [p2(end,:);p2(1,:)], param,xn); %[top, bottom]
            c3 = sxybc - asymmetricRolling.sxy2minus(xrow,asymmetricRolling.ht(xrow,param.rt,param.delta),sxx0(1,:),p0(1,:),param,xn) - asymmetricRolling.ht(xrow,param.rt,param.delta).*dp0c2(2,:);
            dc2 = repmat(dp0c2(2,:), vert, 1);
            c3 = repmat(c3, vert, 1);
            sxy = asymmetricRolling.sxy2minus(x,y,sxx0,p0,param,xn) + dc2.*y + c3;
        end
        function [sxytop, sxybot] = sxy2BC(x, sxx0, p0, sxy0, sxx2, p2, param, xn)
            beta = param.beta/param.delta;
            gammat = param.fricTop(x,xn(1));
            gammab = param.fricBot(x,xn(2)); 
            dht = asymmetricRolling.htprime(x,param.rt,param.delta);
            dhb = asymmetricRolling.hbprime(x,param.rb,param.delta);
            sxytop = gammat.*(beta*p2(1,:) + sxx2(1,:)) + 2/beta*sxx2(1,:).*dht + 2*beta*gammat.*(sxy0(1,:).*dht + p0(1,:).*dht.^2) + 2/beta.*sxx0(1,:).*dht.^3;
            sxybot = gammab.*(beta*p2(2,:) + sxx2(2,:)) + 2/beta*sxx2(2,:).*dhb + 2*beta*gammab.*(sxy0(2,:).*dhb + p0(2,:).*dhb.^2) + 2/beta.*sxx0(2,:).*dhb.^3;
        end


%% Ancillary Functions
        function [value, term, back] = odeSolverTermination(x, p, termFunc, param)
            psym = termFunc(x);
            psym = psym(1) + param.delta.^2.*psym(2);
            value = p(1)+param.delta.^2.*p(2) - psym; %value to cross zero to trigger event
            term = 1; %terminates the condition
            back = 1; %value crosses from below to above
        end
        function [f, t] = ode45Wrapper(func, tspan, tini, fini, opts)
            %Wrapper for ode45 to produce output corresponding to tspan values
            if isempty(tspan)
                t=[];
                f=[];
            else
                t = unique([tini, tspan]); %
                [t,fout] = ode45(func, t, fini, opts);
                if length(tspan(1,:)) == 1
                    f = transpose(fout(end,:));
                    t = t(end);
                elseif length(tspan(1,:)) == 2 && tspan(1,1)==tini
                    f = [transpose(fout(1,:)), transpose(fout(end,:))];
                    t = [t(1), t(end)];
                elseif tspan(1,1) == tini
                    f = transpose(fout);
                    t = transpose(t);
                else
                    f = transpose(fout(2:end,:));
                    t = transpose(t(2:end));
                end
            end
        end

%% 'Post 'scaled solution calculation' functions
        function [rollPressure, rollShear, rollForce, rollTorque, rollForceHorizontal] = rollPressure(res, grid, param)
            %COMPUTEROLLFORCES computes roll pressure, shear, force and torque.
                Rt = param.Rt;
                phi(1,:) = asin((param.wl - grid.X(end,:))/Rt);
                phin(1) = asin((param.wl - res.xn(1))/Rt);
                %rollPressure positive for compression.
                [p, ~, syy, sxy] = asymmetricRolling.mohrsCircle(res.p(end,:), res.sxx(end,:), res.syy(end,:), res.sxy(end,:), phi(1,:));
                rollShear(1,:) = sxy; %roller torque convention
                rollPressure(1,:) = -(-p + syy);  %positive for compression
                [rollForce(1), rollTorque(1), rollForceHorizontal(1)] = asymmetricRolling.rollForces(rollPressure(1,:), rollShear(1,:), phi(1,:), Rt);
                try
                    [shearIn, shearOut, ~, ~] = asymmetricRolling.interpolateNeutralPoint((phi(1,:)), (rollShear(1,:)), phin(1));
                    [pIn, pOut, phiIn, phiOut] = asymmetricRolling.interpolateNeutralPoint((phi(1,:)), (rollPressure(1,:)), phin(1));
                    [~, rTIn, ~] = asymmetricRolling.rollForces(pIn, shearIn, phiIn, Rt);        
                    [~, rTOut, ~] = asymmetricRolling.rollForces(pOut, shearOut, phiOut, Rt);        
                    rollTorque(1) = rTIn+rTOut;
                    clear shearIn shearOut pIn pOut phiIn phiOut rTIn rTOut
                catch e
                    disp(e.identifier)
                    disp(e.message)            
                end
                
                if isfield(param, 'Rb')
                    phi(2,:) = pi - asin((param.wl - grid.X(1,:))/param.Rb);
                    phin(2) = pi - asin((param.wl - res.xn(2))/param.Rb);
                    [p, ~, syy, sxy] = asymmetricRolling.mohrsCircle(res.p(1,:), res.sxx(1,:), res.syy(1,:), res.sxy(1,:), phi(2,:));
                    rollShear(2,:) = sxy; %roller torque convention
                    rollPressure(2,:) =-(-p + syy); %positive for compression
                    [rollForce(2), rollTorque(2), rollForceHorizontal(2)] = asymmetricRolling.rollForces(rollPressure(2,:), rollShear(2,:), phi(2,:), param.Rb);
                    try
                        [shearIn, shearOut, ~, ~] = asymmetricRolling.interpolateNeutralPoint(phi(2,:), rollShear(2,:), phin(2));
                        [pIn, pOut, phiIn, phiOut] = asymmetricRolling.interpolateNeutralPoint(phi(2,:), rollPressure(2,:), phin(2));

                        [~, rTIn, ~] = asymmetricRolling.rollForces(pIn, shearIn, phiIn, param.Rb);        
                        [~, rTOut, ~] = asymmetricRolling.rollForces(pOut, shearOut, phiOut, param.Rb);        
                        rollTorque(2) = rTIn+rTOut;
                        clear shearIn shearOut pIn pOut phiIn phiOut rTIn rTOut
                    catch e
                        disp(e.identifier)
                        disp(e.message)
                    end
                end
        end
        function [f] = horizontalForce(sigmaxx, grid, param, node)
            %COMPUTEHORIZONTALFORCE Computes the horzontal force as tension at
            %each cross-section of the roll gap.
            f = node;
            for i=1:length(node)
                f(i) = trapz(grid.Y(:,node(i)), sigmaxx(:,node(i))); %width(m) * height(m) * pressure(N/m^2 or Pa) = N
            end
        end
        function [ P, Sxx, Syy, Sxy ] = mohrsCircle( p, sxx, syy, sxy, theta )
        %MOHRSCIRCLE Calculates the new stresses after a rotation has occured.
        %    [P, Sxx, Syy, Sxy] = mohrsCircle(p, sxx, syy, sxy, theta)
            for i = 1:length(p)
                sigma = [-p(i)+sxx(i), sxy(i);
                        sxy(i), -p(i)+syy(i)];
                R = [cos(theta(i)), -sin(theta(i));
                    sin(theta(i)), cos(theta(i))];
                Sigma = R*sigma*transpose(R);
                P(i) = -(Sigma(1,1) + Sigma(2,2))/2;
                Sxx(i) = Sigma(1,1) + P(i);
                Syy(i) = Sigma(2,2) + P(i);
                Sxy(i) = Sigma(1,2);
            end
        end
        function [rollForce, rollTorque, rollForceHorizontal] = rollForces(rollPressure, rollShear, phi, rollRadius)
            %Check to make sure phi is ascending and flip if descending
            if ~issorted(phi)
                phi = fliplr(phi);
                assert(issorted(phi));
                rollPressure = fliplr(rollPressure);
                rollShear = fliplr(rollShear);
            end
            %COMPUTEROLLFORCES computes roll forces and torque.
            rollForce = ...
                    trapz(rollRadius*phi, -cos(phi).*rollPressure)+...
                    trapz(rollRadius*phi, sin(phi).*rollShear);
            rollForceHorizontal = ...
                    trapz(rollRadius*phi, -sin(phi).*rollPressure)+...
                    trapz(rollRadius*phi, -cos(phi).*rollShear);
            rollTorque =...
                trapz(rollRadius*phi, rollRadius*rollShear);
        end
        function [valIn, valOut, phiIn, phiOut] = interpolateNeutralPoint(phi, val, phin)
            %Generates two sets of values for either side of the neutral point.
            %Check to make sure phi is ascending and flip if descending
            ascend=true;
            if ~issorted(phi)
                assert(issorted(fliplr(phi)));
                ascend = false;
            end
            indn = length(phi);
            for i=1:length(phi)
                if phi(i)>phin && ascend
                    indn = i-1;
                    break
                elseif phi(i)<phin && ~ascend
                    indn = i-1;
                    break
                end
            end
            if indn==length(phi)
                valIn = val;
                valOut = [];
                phiIn = phi;
                phiOut = [];
            elseif indn==length(phi)-1
                valIn = [val(1:indn), interp1(phi(1:indn),val(1:indn),phin,'linear','extrap')];
                valOut = [val(end), val(end)];
                phiIn = [phi(1:indn), phin];
                phiOut = [phin, phi(end)];
            elseif indn==0;
                valIn = [];
                valOut = val;
                phiIn = [];
                phiOut = phi;
            elseif indn==1
                valIn = [val(1), val(1)];
                valOut = [interp1(phi(indn+1:end),val(indn+1:end),phin,'linear','extrap'), val(indn+1:end)];
                phiIn = [phi(1), phin];
                phiOut = [phin(1), phi(indn+1:end)];
            else
                valIn = [val(1:indn), interp1(phi(1:indn),val(1:indn),phin,'linear','extrap')];
                valOut = [interp1(phi(indn+1:end),val(indn+1:end),phin,'linear','extrap'), val(indn+1:end)];
                phiIn = [phi(1:indn), phin];
                phiOut = [phin(1), phi(indn+1:end)];
            end
        end
    end
end

